@@ -1,16 +1,16 @@
 ;;; esh-var.el --- handling of variables
 
 ;; Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
 ;; Author: John Wiegley <johnw@gnu.org>
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,21 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
-(provide 'esh-var)
-
-(eval-when-compile (require 'esh-maint))
-
-(defgroup eshell-var nil
-  "Variable interpolation is introduced whenever the '$' character
-appears unquoted in any argument (except when that argument is
-surrounded by single quotes).  It may be used to interpolate a
-variable value, a subcommand, or even the result of a Lisp form."
-  :tag "Variable handling"
-  :group 'eshell)
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -47,7 +33,7 @@ variable value, a subcommand, or even the result of a Lisp form."
 ;;
 ;;   $ALSO-VAR
 ;;
-;; "-" is a legal part of a variable name.
+;; "-" is a valid part of a variable name.
 ;;
 ;;   $<MYVAR>-TOO
 ;;
@@ -118,9 +104,27 @@ variable value, a subcommand, or even the result of a Lisp form."
 ;; contains the exit code of the last command (0 or 1 for Lisp
 ;; functions, based on successful completion).
 
+;;; Code:
+
+(provide 'esh-var)
+
+(eval-when-compile
+  (require 'pcomplete)
+  (require 'esh-test)
+  (require 'esh-util)
+  (require 'esh-opt)
+  (require 'esh-mode))
 (require 'env)
 (require 'ring)
 
+(defgroup eshell-var nil
+  "Variable interpolation is introduced whenever the '$' character
+appears unquoted in any argument (except when that argument is
+surrounded by single quotes).  It may be used to interpolate a
+variable value, a subcommand, or even the result of a Lisp form."
+  :tag "Variable handling"
+  :group 'eshell)
+
 ;;; User Variables:
 
 (defcustom eshell-var-load-hook '(eshell-var-initialize)
@@ -649,7 +653,5 @@ For example, to retrieve the second element of a user's record in
       (setq pcomplete-stub (substring arg pos))
       (throw 'pcomplete-completions (pcomplete-entries)))))
 
-;;; Code:
-
-;;; arch-tag: 393654fe-bdad-4f27-9a10-b1472ded14cf
+;; arch-tag: 393654fe-bdad-4f27-9a10-b1472ded14cf
 ;;; esh-var.el ends here
