@@ -1,7 +1,7 @@
 /* The lwlib interface to Athena widgets.
 Copyright (C) 1993 Chuck Thompson <cthomp@cs.uiuc.edu>
 Copyright (C) 1994, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2007, 2008, 2009 Free Software Foundation, Inc.
 
 This file is part of the Lucid Widget Library.
 
@@ -25,7 +25,6 @@ Boston, MA 02110-1301, USA.  */
 #endif
 
 #include <stdio.h>
-#include <setjmp.h>
 
 #include "../src/lisp.h"
 
