@@ -1,7 +1,7 @@
 /*
 Copyright (C) 1992, 1993 Lucid, Inc.
 Copyright (C) 1994, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009, 2010  Free Software Foundation, Inc.
+  2007, 2008, 2009  Free Software Foundation, Inc.
 
 This file is part of the Lucid Widget Library.
 
