@@ -1,6 +1,6 @@
 /* Proxy shell designed for use with Emacs on Windows 95 and NT.
    Copyright (C) 1997, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-     2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+     2008, 2009  Free Software Foundation, Inc.
 
    Accepts subset of Unix sh(1) command-line options, for compatibility
    with elisp code written for Unix.  When possible, executes external
@@ -35,9 +35,6 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 #include <stdlib.h>  /* getenv */
 #include <string.h>  /* strlen */
 
-/* We don't want to include stdio.h because we are already duplicating
-   lots of it here */
-extern int _snprintf (char *buffer, size_t count, const char *format, ...);
 
 /*******  Mock C library routines  *********************************/
 
@@ -607,7 +604,6 @@ main (int argc, char ** argv)
     {
       char * p;
       int    extra_arg_space = 0;
-      int    maxlen, remlen;
       int    run_command_dot_com;
 
       progname = getenv ("COMSPEC");
@@ -639,27 +635,21 @@ main (int argc, char ** argv)
 	     case path contains spaces (fortunately it can't contain
 	     quotes, since they are illegal in path names).  */
 
-	  remlen = maxlen =
-	    strlen (progname) + extra_arg_space + strlen (cmdline) + 16;
-	  buf = p = alloca (maxlen + 1);
+	  buf = p = alloca (strlen (progname) + extra_arg_space +
+			    strlen (cmdline) + 16);
 
 	  /* Quote progname in case it contains spaces.  */
-	  p += _snprintf (p, remlen, "\"%s\"", progname);
-	  remlen = maxlen - (p - buf);
+	  p += wsprintf (p, "\"%s\"", progname);
 
 	  /* Include pass_through_args verbatim; these are just switches
              so should not need quoting.  */
 	  for (argv = pass_through_args; *argv != NULL; ++argv)
-	    {
-	      p += _snprintf (p, remlen, " %s", *argv);
-	      remlen = maxlen - (p - buf);
-	    }
+	    p += wsprintf (p, " %s", *argv);
 
 	  if (run_command_dot_com)
-	    _snprintf (p, remlen, " /e:%d /c %s", envsize, cmdline);
+	    wsprintf(p, " /e:%d /c %s", envsize, cmdline);
 	  else
-	    _snprintf (p, remlen, " /c %s", cmdline);
-	  remlen = maxlen - (p - buf);
+	    wsprintf(p, " /c %s", cmdline);
 	  cmdline = buf;
 	}
       else
@@ -679,27 +669,19 @@ main (int argc, char ** argv)
 	  else
 	    path[0] = '\0';
 
-	  remlen = maxlen =
-	    strlen (progname) + extra_arg_space + strlen (path) + 13;
-	  cmdline = p = alloca (maxlen + 1);
+	  cmdline = p = alloca (strlen (progname) + extra_arg_space +
+				strlen (path) + 13);
 
 	  /* Quote progname in case it contains spaces.  */
-	  p += _snprintf (p, remlen, "\"%s\" %s", progname, path);
-	  remlen = maxlen - (p - cmdline);
+	  p += wsprintf (p, "\"%s\" %s", progname, path);
 
 	  /* Include pass_through_args verbatim; these are just switches
              so should not need quoting.  */
 	  for (argv = pass_through_args; *argv != NULL; ++argv)
-	    {
-	      p += _snprintf (p, remlen, " %s", *argv);
-	      remlen = maxlen - (p - cmdline);
-	    }
+	    p += wsprintf (p, " %s", *argv);
 
 	  if (run_command_dot_com)
-	    {
-	      _snprintf (p, remlen, " /e:%d", envsize);
-	      remlen = maxlen - (p - cmdline);
-	    }
+	    wsprintf (p, " /e:%d", envsize);
 	}
     }
 
