@@ -1,7 +1,7 @@
 /* Header for multibyte character handler.
    Copyright (C) 1995, 1997, 1998 Electrotechnical Laboratory, JAPAN.
      Licensed to the Free Software Foundation.
-   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H13PRO009
 
@@ -62,9 +62,6 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 /* Return the character code for raw 8-bit byte BYTE.  */
 #define BYTE8_TO_CHAR(byte) ((byte) + 0x3FFF00)
 
-#define UNIBYTE_TO_CHAR(byte) \
-  (ASCII_BYTE_P (byte) ? (byte) : BYTE8_TO_CHAR (byte))
-
 /* Return the raw 8-bit byte for character C.  */
 #define CHAR_TO_BYTE8(c)	\
   (CHAR_BYTE8_P (c)		\
@@ -82,6 +79,24 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
    that corresponds to a raw 8-bit byte.  */
 #define CHAR_BYTE8_HEAD_P(byte) ((byte) == 0xC0 || (byte) == 0xC1)
 
+/* Mapping table from unibyte chars to multibyte chars.  */
+extern int unibyte_to_multibyte_table[256];
+
+/* Convert the unibyte character C to the corresponding multibyte
+   character.  If C can't be converted, return C.  */
+#define unibyte_char_to_multibyte(c)	\
+  ((c) < 256 ? unibyte_to_multibyte_table[(c)] : (c))
+
+/* Decoding table for 8-bit byte codes of the charset charset_unibyte.
+   Nth element is for the code (N+0x80).  */
+extern int charset_unibyte_decoder[128];
+
+/* Return a character correspoinding to the code BYTE of
+   charset_unibyte.  BYTE must be a byte; i.e. less than 0x100.  If
+   BYTE is not a valid code of charset_unibyte, return -1.  */
+#define DECODE_UNIBYTE(BYTE)	\
+  ((BYTE) < 0x80 ? (int) (BYTE) : charset_unibyte_decoder[(BYTE) - 0x80])
+
 /* If C is not ASCII, make it unibyte. */
 #define MAKE_CHAR_UNIBYTE(c)	\
   do {				\
@@ -92,7 +107,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* If C is not ASCII, make it multibyte.  Assumes C < 256.  */
 #define MAKE_CHAR_MULTIBYTE(c) \
-  (eassert ((c) >= 0 && (c) < 256), (c) = UNIBYTE_TO_CHAR (c))
+  (eassert ((c) >= 0 && (c) < 256), (c) = unibyte_to_multibyte_table[(c)])
 
 /* This is the maximum byte length of multibyte form.  */
 #define MAX_MULTIBYTE_LENGTH 5
@@ -136,8 +151,8 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Nonzero if character C has a printable glyph.  */
 #define CHAR_PRINTABLE_P(c)	\
-  (((c) >= 32 && (c) < 127)	\
-   || ! NILP (CHAR_TABLE_REF (Vprintable_chars, (c))))
+  (((c) >= 32 && ((c) < 127)	\
+    || ! NILP (CHAR_TABLE_REF (Vprintable_chars, (c)))))
 
 /* Return byte length of multibyte form for character C.  */
 #define CHAR_BYTES(c)			\
@@ -311,9 +326,10 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
   } while (0)
 
 /* Return the character code of character whose multibyte form is at
-   P.  */
+   P.  The argument LEN is ignored.  It will be removed in the
+   future.  */
 
-#define STRING_CHAR(p)						\
+#define STRING_CHAR(p, len)					\
   (!((p)[0] & 0x80)						\
    ? (p)[0]							\
    : ! ((p)[0] & 0x20)						\
@@ -328,9 +344,10 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 /* Like STRING_CHAR, but set ACTUAL_LEN to the length of multibyte
-   form.  */
+   form.  The argument LEN is ignored.  It will be removed in the
+   future.  */
 
-#define STRING_CHAR_AND_LENGTH(p, actual_len)			\
+#define STRING_CHAR_AND_LENGTH(p, len, actual_len)		\
   (!((p)[0] & 0x80)						\
    ? ((actual_len) = 1, (p)[0])					\
    : ! ((p)[0] & 0x20)						\
@@ -380,7 +397,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 	  unsigned char *ptr = &SDATA (STRING)[BYTEIDX];		\
 	  int len;							\
 									\
-	  OUTPUT = STRING_CHAR_AND_LENGTH (ptr, len);			\
+	  OUTPUT = STRING_CHAR_AND_LENGTH (ptr, 0, len);		\
 	  BYTEIDX += len;						\
 	}								\
       else								\
@@ -403,7 +420,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 	  unsigned char *ptr = &SDATA (STRING)[BYTEIDX];		      \
 	  int len;							      \
 									      \
-	  OUTPUT = STRING_CHAR_AND_LENGTH (ptr, len);			      \
+	  OUTPUT = STRING_CHAR_AND_LENGTH (ptr, 0, len);		      \
 	  BYTEIDX += len;						      \
 	}								      \
       else								      \
@@ -424,7 +441,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
       unsigned char *ptr = &SDATA (STRING)[BYTEIDX];			     \
       int len;								     \
 									     \
-      OUTPUT = STRING_CHAR_AND_LENGTH (ptr, len);			     \
+      OUTPUT = STRING_CHAR_AND_LENGTH (ptr, 0, len);			     \
       BYTEIDX += len;							     \
       CHARIDX++;							     \
     }									     \
@@ -443,7 +460,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 	  unsigned char *ptr = BYTE_POS_ADDR (BYTEIDX);		\
 	  int len;						\
 								\
-	  OUTPUT= STRING_CHAR_AND_LENGTH (ptr, len);		\
+	  OUTPUT= STRING_CHAR_AND_LENGTH (ptr, 0, len);		\
 	  BYTEIDX += len;					\
 	}							\
       else							\
@@ -463,7 +480,7 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
       unsigned char *ptr = BYTE_POS_ADDR (BYTEIDX);		\
       int len;							\
 								\
-      OUTPUT = STRING_CHAR_AND_LENGTH (ptr, len);		\
+      OUTPUT= STRING_CHAR_AND_LENGTH (ptr, 0, len);		\
       BYTEIDX += len;						\
       CHARIDX++;						\
     }								\
@@ -670,7 +687,7 @@ extern Lisp_Object Vscript_representative_chars;
   } while (0)
 
 #define DEFSYM(sym, name)	\
-  do { (sym) = intern_c_string ((name)); staticpro (&(sym)); } while (0)
+  do { (sym) = intern ((name)); staticpro (&(sym)); } while (0)
 
 #endif /* EMACS_CHARACTER_H */
 
